@@ -24,6 +24,7 @@
 
 #include "nm-vpnc-editor-plugin.h"
 
+#include <gmodule.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
@@ -817,23 +818,35 @@ _call_editor_factory (gpointer factory,
 static NMVpnEditor *
 get_editor (NMVpnEditorPlugin *iface, NMConnection *connection, GError **error)
 {
+	gpointer gtk3_only_symbol;
+	GModule *self_module;
+	const char *editor;
+
 	g_return_val_if_fail (VPNC_IS_EDITOR_PLUGIN (iface), NULL);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), NULL);
 	g_return_val_if_fail (!error || !*error, NULL);
 
-	{
-#ifdef NM_VPN_OLD
-		return nm_vpnc_editor_new (connection, error);
-#else
-		return nm_vpn_plugin_utils_load_editor (NM_PLUGIN_DIR"/libnm-vpn-plugin-vpnc-editor.so",
-		                                        "nm_vpn_editor_factory_vpnc",
-		                                        _call_editor_factory,
-		                                        iface,
-		                                        connection,
-		                                        NULL,
-		                                        error);
-#endif
+	self_module = g_module_open (NULL, 0);
+	g_module_symbol (self_module, "gtk_container_add", &gtk3_only_symbol);
+	g_module_close (self_module);
+
+	if (gtk3_only_symbol) {
+		editor = "libnm-vpn-plugin-vpnc-editor.so";
+	} else {
+		editor = "libnm-gtk4-vpn-plugin-vpnc-editor.so";
 	}
+
+#ifdef NM_VPN_OLD
+	return nm_vpnc_editor_new (connection, error);
+#else
+	return nm_vpn_plugin_utils_load_editor (editor,
+						"nm_vpn_editor_factory_vpnc",
+						_call_editor_factory,
+						iface,
+						connection,
+						NULL,
+						error);
+#endif
 }
 
 static void
